includeTargets << grailsScript("_GrailsArgParsing")
includeTargets << grailsScript("_GrailsCreateArtifacts")
includeTargets << new File("${smartgwtServerPluginDir}/scripts/_SmartGwtCreate.groovy")

USAGE = """
    create-smartgwt-module PKG.NAME

where
    PKG  = The package name of the module.
    NAME = The name of the module.
"""

target (default: "Creates a new SmartGWT module.") {
    depends(parseArguments)
    promptForName(type: "")

    // We support just the one argument.
    def params = argsMap["params"]
    if (!params || params.size() > 1) {
        println "Unexpected number of command arguments."
        println()
        println "USAGE:${USAGE}"
        exit(1)
    }
    else if (!params[0]) {
        println "A module name must be given."
        exit(1)
    }
    
    // We must split the argument into package and name parts.
    def (modulePackage, moduleName) = packageAndName(params[0])
    
    // We require a package for the module.
    if (!modulePackage) {
        println "Please provide a package for the module."
        exit(1)
    }

	// We copy a server.properties file if one does not exist
	if (!new File("${basedir}/src/java/server.properties").exists()) {
		def targetFile = new File("${basedir}/src/java", "server.properties")
		def templateFile = new File("${smartgwtServerPluginDir}/src/templates/artifacts", "server.properties")
		installFile(targetFile, templateFile, [ "artifact.package": modulePackage, "artifact.name": moduleName ])
	}

	// We copy a log4j.isc.config.xml file if one does not exist
	if (!new File("${basedir}/src/java/log4j.isc.config.xml").exists()) {
		def targetFile = new File("${basedir}/src/java", "log4j.isc.config.xml")
		def templateFile = new File("${smartgwtServerPluginDir}/src/templates/artifacts", "log4j.isc.config.xml")
		installFile(targetFile, templateFile, [ "artifact.package": modulePackage, "artifact.name": moduleName ])
	}

    // Now create the module XML file.
    installSmartGwtTemplate(modulePackage, moduleName, "SmartGwt.gwt.xml")

    // Now copy the template client entry point over.
    installSmartGwtTemplate(modulePackage + ".client", moduleName, "SmartGwt.java")
}
