/*
 */

import groovy.xml.MarkupBuilder
import org.codehaus.groovy.grails.commons.GrailsDomainClassProperty
import org.codehaus.groovy.grails.commons.GrailsDomainClass
import grails.persistence.Event
import com.emrul.grails.smartclient.GrailsSmartClientDataSourceUtil
import com.emrul.grails.smartclient.GrailsSmartClientSharedUtil
import com.emrul.grails.smartclient.GrailsSmartClientUtil

Ant.property(environment:"env")
grailsHome = Ant.antProject.properties."env.GRAILS_HOME"

includeTargets << grailsScript("_GrailsArgParsing")
includeTargets << grailsScript("_GrailsCreateArtifacts")
includeTargets << new File("${smartgwtServerPluginDir}/scripts/_SmartGwtCreate.groovy")
includeTargets << new File ( "${grailsHome}/scripts/Bootstrap.groovy" )

USAGE = """
    generate-scaffolding GSPFILE PKG.NAME

where
    GSPFILE = The name of the GSP file to create
    PKG  = The package name of the module.
    NAME = The name of the module.

e.g.
    generate-scaffolding smartgwt.gsp com.example.smartgwt.Scaffolding
"""

target(main: "Generates SmartGWT Scaffolding") {
	depends( checkVersion, parseArguments, packageApp, loadApp)

    // Two arguments required.
    def params = argsMap["params"]
    if (!params || params.size() > 2) {
        println "Unexpected number of command arguments."
        println()
        println "USAGE:${USAGE}"
        exit(1)
    }
    else if (!params[0]) {
        println "A gsp file name must be given."
        exit(1)
    }
    else if (!params[1]) {
        println "A module name must be given."
        exit(1)
    }

    // We must split the argument into package and name parts.
    def (modulePackage, moduleName) = packageAndName(params[1])

    // We require a package for the module.
    if (!modulePackage) {
        println "Please provide a package for the module."
        exit(1)
    }

	// We copy a server.properties file if one does not exist
	if (!new File("${basedir}/src/java/server.properties").exists()) {
		def targetFile = new File("${basedir}/src/java", "server.properties")
		def templateFile = new File("${smartgwtServerPluginDir}/src/templates/artifacts", "server.properties")
		installFile(targetFile, templateFile, [ "artifact.package": modulePackage, "artifact.name": moduleName ])
	}

    // We copy a log4j.isc.config.xml file if one does not exist
    if (!new File("${basedir}/src/java/log4j.isc.config.xml").exists()) {
        def targetFile = new File("${basedir}/src/java", "log4j.isc.config.xml")
        def templateFile = new File("${smartgwtServerPluginDir}/src/templates/artifacts", "log4j.isc.config.xml")
        installFile(targetFile, templateFile, [ "artifact.package": modulePackage, "artifact.name": moduleName ])
    }
    ///////// Generate scaffold module
	println 'Generating GWT scaffolding module'

    // Now create the module Xml file.
    installSmartGwtTemplate(modulePackage, moduleName, "SmartGwt.gwt.xml")
    // Now copy the template client entry point over.
    installSmartGwtScaffoldTemplate(modulePackage + ".client", moduleName, "SmartGwtScaffold.java")

    ///////// Generate host page for scaffolding
	println 'Generating GWT scaffolding host page'

    // Location of the templates.
    def templatePath = "${smartgwtServerPluginDir}/src/templates/artifacts"
    def scaffoldHostPageTemplateFile = "${templatePath}/SmartGwtScaffoldHostPage.tmpl"

    // Copy the template file to the target location.
    targetHostPageFile = "grails-app/views/${params[0]}"
    ant.copy(file: scaffoldHostPageTemplateFile, tofile: targetHostPageFile, overwrite: true)

    // Replace the tokens in the template.
    ant.replace(file: targetHostPageFile) {
        replacefilter(token: '@module.name@', value: params[1])
    }

    ///////// Generate SmartGWT widgets, datasources and an AppDomainInfo DataSource
	println 'Generating scaffolding'

	//def GrailsSmartClientDataSourceUtil = grailsApp.classLoader.loadClass("com.emrul.grails.smartclient.GrailsSmartClientDataSourceUtil")
	//def GrailsSmartClientSharedUtil = grailsApp.classLoader.loadClass("com.emrul.grails.smartclient.GrailsSmartClientSharedUtil")

	def datasourceDir = new File("${basedir}/smartclient/ds")
	def testDataDir = new File("${datasourceDir}/test_data")
	if (!testDataDir.exists())
		testDataDir.mkdirs()
	def dsWriter = new FileWriter( new File("${datasourceDir}/_AppDomainInfo.ds.groovy") )
	def dsDataWriter = new FileWriter( new File("${testDataDir}/_AppDomainInfo.data.groovy") )
	GrailsSmartClientDataSourceUtil.generateAppDomainInfoDS(grailsApp.domainClasses, dsWriter, dsDataWriter )
	dsWriter.close()
	dsDataWriter.close()

	//def GrailsSmartClientUtil = grailsApp.classLoader.loadClass("com.emrul.grails.smartclient.GrailsSmartClientUtil")

	def widgetsDirPath = "${basedir}/smartclient/ui"
	def widgetsDir = new File(widgetsDirPath)
	if (!widgetsDir.exists())
		widgetsDir.mkdirs()

    def scaffoldUITemplateFile = "${templatePath}/Z_ScaffoldApp.ui.xml"
    ant.copy(file: scaffoldUITemplateFile, todir: widgetsDir, overwrite: true)

	GrailsSmartClientUtil.generateAllUIWidgets( grailsApp.domainClasses, widgetsDirPath )
}
setDefaultTarget(main)
